@@ -1,24 +1,16 @@
 import re
-import requests
 import json
+import requests
+import urllib.parse as urlparse
 
-try:
-    # Python <= 2.7
-    import urlparse
-    from urllib import urlencode
-except ImportError:
-    # support for py3
-    import urllib.parse as urlparse
-    from urllib.parse import urlencode
-
-from django.conf import settings
+from django.http import QueryDict
 from django.template.loader import render_to_string
 from django.utils.functional import cached_property
 from django.utils.safestring import mark_safe
-from django.utils.datastructures import SortedDict
 
 from .utils import import_by_path
-from .settings import EMBED_VIDEO_BACKENDS, EMBED_VIDEO_TIMEOUT
+from .settings import EMBED_VIDEO_BACKENDS, EMBED_VIDEO_TIMEOUT, \
+    EMBED_VIDEO_YOUTUBE_DEFAULT_QUERY
 
 
 class EmbedVideoException(Exception):
@@ -49,6 +41,12 @@ def detect_backend(url):
 
     Goes over backends in ``settings.EMBED_VIDEO_BACKENDS``,
     calls :py:func:`~VideoBackend.is_valid` and returns backend instance.
+
+    :param url: URL which is passed to `is_valid` methods of VideoBackends.
+    :type url: str
+
+    :return: Returns recognized VideoBackend
+    :rtype: VideoBackend
     """
 
     for backend_name in EMBED_VIDEO_BACKENDS:
@@ -59,10 +57,25 @@ def detect_backend(url):
     raise UnknownBackendException
 
 
-class VideoBackend(object):
+class VideoBackend:
     """
     Base class used as parental class for backends.
 
+
+    Backend variables:
+
+    .. autosummary::
+
+        url
+        code
+        thumbnail
+        query
+        info
+        is_secure
+        protocol
+        template_name
+
+
     .. code-block:: python
 
         class MyBackend(VideoBackend):
@@ -90,6 +103,8 @@ class VideoBackend(object):
     Pattern in which the code is inserted.
 
     Example: ``http://myvideo.com?code=%s``
+
+    :type: str
     """
 
     pattern_thumbnail_url = None
@@ -97,11 +112,15 @@ class VideoBackend(object):
     Pattern in which the code is inserted to get thumbnail url.
 
     Example: ``http://static.myvideo.com/thumbs/%s``
+
+    :type: str
     """
 
     allow_https = True
     """
     Sets if HTTPS version allowed for specific backend.
+
+    :type: bool
     """
 
     template_name = 'embed_video/embed_code.html'
@@ -110,54 +129,101 @@ class VideoBackend(object):
 
     Passed template variables: ``{{ backend }}`` (instance of VideoBackend),
     ``{{ width }}``, ``{{ height }}``
+
+    :type: str
     """
 
-    def __init__(self, url, is_secure=False, query=None):
+    default_query = ''
+    """
+    Default query string or `QueryDict` appended to url
+
+    :type: str
+    """
+
+    is_secure = False
+    """
+    Decides if secured protocol (HTTPS) is used.
+
+    :type: bool
+    """
+
+    def __init__(self, url):
         """
         First it tries to load data from cache and if it don't succeed, run
         :py:meth:`init` and then save it to cache.
+
+        :type url: str
         """
-        self.is_secure = is_secure
         self.backend = self.__class__.__name__
         self._url = url
-        self.update_query(query)
+        self.query = QueryDict(self.default_query, mutable=True)
 
-    def update_query(self, query=None):
-        self._query = SortedDict(self.get_default_query())
-        if query is not None:
-            self._query.update(query)
-
-    @cached_property
+    @property
     def code(self):
+        """
+        Code of video.
+        """
         return self.get_code()
 
-    @cached_property
+    @property
     def url(self):
+        """
+        URL of video.
+        """
         return self.get_url()
 
-    @cached_property
+    @property
     def protocol(self):
+        """
+        Protocol used to generate URL.
+        """
         return 'https' if self.allow_https and self.is_secure else 'http'
 
-    @cached_property
+    @property
     def thumbnail(self):
+        """
+        URL of video thumbnail.
+        """
         return self.get_thumbnail_url()
 
-    @cached_property
+    @property
     def info(self):
+        """
+        Additional information about video. Not implemented in all backends.
+        """
         return self.get_info()
 
+    @property
+    def query(self):
+        """
+        String transformed to QueryDict appended to url.
+        """
+        return self._query
+
+    @query.setter
+    def query(self, value):
+        """
+        :type value: QueryDict | str
+        """
+        self._query = value \
+            if isinstance(value, QueryDict) \
+            else QueryDict(value, mutable=True)
+
     @classmethod
     def is_valid(cls, url):
         """
         Class method to control if passed url is valid for current backend. By
         default it is done by :py:data:`re_detect` regex.
+
+        :type url: str
         """
         return True if cls.re_detect.match(url) else False
 
     def get_code(self):
         """
         Returns video code matched from given url by :py:data:`re_code`.
+
+        :rtype: str
         """
         match = self.re_code.search(self._url)
         if match:
@@ -168,14 +234,15 @@ class VideoBackend(object):
         Returns URL folded from :py:data:`pattern_url` and parsed code.
         """
         url = self.pattern_url.format(code=self.code, protocol=self.protocol)
-        if self._query:
-            url += '?' + urlencode(self._query, doseq=True)
+        url += '?' + self.query.urlencode() if self.query else ''
         return mark_safe(url)
 
     def get_thumbnail_url(self):
         """
         Returns thumbnail URL folded from :py:data:`pattern_thumbnail_url` and
         parsed code.
+
+        :rtype: str
         """
         return self.pattern_thumbnail_url.format(code=self.code,
                                                  protocol=self.protocol)
@@ -183,6 +250,10 @@ class VideoBackend(object):
     def get_embed_code(self, width, height):
         """
         Returns embed code rendered from template :py:data:`template_name`.
+
+        :type width: int | str
+        :type height: int | str
+        :rtype: str
         """
         return render_to_string(self.template_name, {
             'backend': self,
@@ -191,14 +262,17 @@ class VideoBackend(object):
         })
 
     def get_info(self):
+        """
+        :rtype: dict
+        """
         raise NotImplementedError
 
-    def get_default_query(self):
-        # Derive backend name from class name
-        backend_name = self.__class__.__name__[:-7].upper()
-        default = getattr(self, 'default_query', {})
-        settings_key = 'EMBED_VIDEO_{0}_QUERY'.format(backend_name)
-        return getattr(settings, settings_key, default).copy()
+    def set_options(self, options):
+        """
+        :type options: dict
+        """
+        for key in options:
+            setattr(self, key, options[key])
 
 
 class YoutubeBackend(VideoBackend):
@@ -224,11 +298,25 @@ class YoutubeBackend(VideoBackend):
     )
 
     pattern_url = '{protocol}://www.youtube.com/embed/{code}'
-    pattern_thumbnail_url = '{protocol}://img.youtube.com/vi/{code}/hqdefault.jpg'
-    default_query = {'wmode': 'opaque'}
+    pattern_thumbnail_url = '{protocol}://img.youtube.com/vi/{code}/{resolution}'
+    default_query = EMBED_VIDEO_YOUTUBE_DEFAULT_QUERY
+    resolutions = [
+        'maxresdefault.jpg',
+        'sddefault.jpg',
+        'hqdefault.jpg',
+        'mqdefault.jpg',
+    ]
+
+    is_secure = True
+    """
+    Decides if secured protocol (HTTPS) is used.
+
+    :type: bool
+    """
+
 
     def get_code(self):
-        code = super(YoutubeBackend, self).get_code()
+        code = super().get_code()
 
         if not code:
             parsed_url = urlparse.urlparse(self._url)
@@ -243,6 +331,20 @@ class YoutubeBackend(VideoBackend):
 
         return code
 
+    def get_thumbnail_url(self):
+        """
+        Returns thumbnail URL folded from :py:data:`pattern_thumbnail_url` and
+        parsed code.
+
+        :rtype: str
+        """
+        for resolution in self.resolutions:
+            temp_thumbnail_url = self.pattern_thumbnail_url.format(
+                code=self.code, protocol=self.protocol, resolution=resolution)
+            if int(requests.head(temp_thumbnail_url).status_code) < 400:
+                return temp_thumbnail_url
+        return None
+
 
 class VimeoBackend(VideoBackend):
     """
@@ -251,14 +353,23 @@ class VimeoBackend(VideoBackend):
     re_detect = re.compile(
         r'^((http(s)?:)?//)?(www\.)?(player\.)?vimeo\.com/.*', re.I
     )
-    re_code = re.compile(r'''vimeo\.com/(video/)?(?P<code>[0-9]+)''', re.I)
+    re_code = re.compile(r'''vimeo\.com/(video/)?(channels/(.*/)?)?(?P<code>[0-9]+)''', re.I)
     pattern_url = '{protocol}://player.vimeo.com/video/{code}'
     pattern_info = '{protocol}://vimeo.com/api/v2/video/{code}.json'
 
+    is_secure = True
+    """
+    Decides if secured protocol (HTTPS) is used.
+
+    :type: bool
+    """
+
+
     def get_info(self):
         try:
             response = requests.get(
-                self.pattern_info.format(code=self.code, protocol=self.protocol),
+                self.pattern_info.format(code=self.code,
+                                         protocol=self.protocol),
                 timeout=EMBED_VIDEO_TIMEOUT
             )
             return json.loads(response.text)[0]
@@ -273,18 +384,31 @@ class SoundCloudBackend(VideoBackend):
     """
     Backend for SoundCloud URLs.
     """
-    base_url = 'http://soundcloud.com/oembed'
+    base_url = '{protocol}://soundcloud.com/oembed'
 
     re_detect = re.compile(r'^(http(s)?://(www\.)?)?soundcloud\.com/.*', re.I)
     re_code = re.compile(r'src=".*%2F(?P<code>\d+)&show_artwork.*"', re.I)
     re_url = re.compile(r'src="(?P<url>.*?)"', re.I)
 
+    is_secure = True
+    """
+    Decides if secured protocol (HTTPS) is used.
+
+    :type: bool
+    """
+
     @cached_property
     def width(self):
+        """
+        :rtype: str
+        """
         return self.info.get('width')
 
     @cached_property
     def height(self):
+        """
+        :rtype: str
+        """
         return self.info.get('height')
 
     def get_info(self):
@@ -292,7 +416,8 @@ class SoundCloudBackend(VideoBackend):
             'format': 'json',
             'url': self._url,
         }
-        r = requests.get(self.base_url, params=params,
+        r = requests.get(self.base_url.format(protocol=self.protocol),
+                         params=params,
                          timeout=EMBED_VIDEO_TIMEOUT)
 
         if r.status_code != 200:
@@ -314,5 +439,4 @@ class SoundCloudBackend(VideoBackend):
         return match.group('code')
 
     def get_embed_code(self, width, height):
-        return super(SoundCloudBackend, self). \
-            get_embed_code(width=width, height=self.height)
+        return super().get_embed_code(width=width, height=height)
