@@ -10,23 +10,14 @@ __all__ = ('EmbedVideoField', 'EmbedVideoFormField')
 
 class EmbedVideoField(models.URLField):
     """
-    Model field for embeded video. Descendant of
+    Model field for embedded video. Descendant of
     :py:class:`django.db.models.URLField`.
     """
 
     def formfield(self, **kwargs):
         defaults = {'form_class': EmbedVideoFormField}
         defaults.update(kwargs)
-        return super(EmbedVideoField, self).formfield(**defaults)
-
-    def south_field_triple(self):
-        from south.modelsinspector import introspector
-        cls_name = '%s.%s' % (
-            self.__class__.__module__,
-            self.__class__.__name__
-        )
-        args, kwargs = introspector(self)
-        return (cls_name, args, kwargs)
+        return super().formfield(**defaults)
 
 
 class EmbedVideoFormField(forms.URLField):
@@ -37,13 +28,14 @@ class EmbedVideoFormField(forms.URLField):
 
     def validate(self, url):
         # if empty url is not allowed throws an exception
-        super(EmbedVideoFormField, self).validate(url)
-        
+        super().validate(url)
+
         if not url:
             return
 
         try:
-            detect_backend(url)
+            backend = detect_backend(url)
+            backend.get_code()
         except UnknownBackendException:
             raise forms.ValidationError(_(u'URL could not be recognized.'))
         except UnknownIdException:
