@@ -1,10 +1,11 @@
-from mock import patch
 from unittest import TestCase
+from unittest.mock import patch
 
 from django.forms import ValidationError
 
 from ..fields import EmbedVideoField, EmbedVideoFormField
-from ..backends import UnknownBackendException, UnknownIdException
+from ..backends import UnknownBackendException, UnknownIdException, \
+    YoutubeBackend
 
 
 class EmbedVideoFieldTestCase(TestCase):
@@ -15,26 +16,19 @@ class EmbedVideoFieldTestCase(TestCase):
         self.assertIsInstance(self.field.formfield(),
                               EmbedVideoFormField)
 
-    def test_south(self):
-        self.assertEqual(self.field.south_field_triple(),
-                         (
-                             'embed_video.fields.EmbedVideoField',
-                             [], {'max_length': '200'}
-                         ))
-
 
 class EmbedVideoFormFieldTestCase(TestCase):
     def setUp(self):
         self.formfield = EmbedVideoFormField()
 
-    def test_validation_unknownbackend(self):
+    def test_validation_unknown_backend(self):
         with patch('embed_video.fields.detect_backend') as mock_detect_backend:
             mock_detect_backend.return_value = True
             mock_detect_backend.side_effect = UnknownBackendException
             self.assertRaises(ValidationError, self.formfield.validate,
                               ('http://youtube.com/v/123/',))
 
-    def test_validation_unknownid(self):
+    def test_validation_unknown_id(self):
         with patch('embed_video.fields.detect_backend') as mock_detect_backend:
             mock_detect_backend.return_value = True
             mock_detect_backend.side_effect = UnknownIdException
@@ -42,11 +36,15 @@ class EmbedVideoFormFieldTestCase(TestCase):
                               ('http://youtube.com/v/123/',))
 
     def test_validation_correct(self):
-        url = 'http://my-testing.url.com'
+        url = 'http://www.youtube.com/watch?v=gauN0gzxTcU'
         with patch('embed_video.fields.detect_backend') as mock_detect_backend:
-            mock_detect_backend.return_value = True
+            mock_detect_backend.return_value = YoutubeBackend(url)
             self.assertEqual(url, self.formfield.validate(url))
 
+    def test_validation_unknown_code(self):
+        url = 'http://www.youtube.com/edit?abcd=abcd'
+        self.assertRaises(ValidationError, self.formfield.validate, url)
+
     def test_validation_super(self):
         self.assertRaises(ValidationError, self.formfield.validate, '')
 
